﻿using System;
using System.IO;

using DateTimeISO;

/////////////////////////////////////////////////////////////////////////
//
// generic common console logging/diagnostic type functions
//
/////////////////////////////////////////////////////////////////////////
namespace ConsoleLog
{
	public class Console_Log
	{
		StreamWriter log_stream;
		string directory_name;
		string program_name;
		DateTime current_date;

		// Constructors

		/// <summary>
		/// Append to program/date specific log file
		/// </summary>
		public Console_Log ()
		{
			InitLog (AppDomain.CurrentDomain.BaseDirectory);
		}

		/// <summary>
		/// Append to program/date specific log file
		/// </summary>
		/// <param name="directory">Alternate directory location</param>
		public Console_Log (string directory)
		{
			InitLog (directory);
		}

		// Public Methods

		/// <summary>
		/// Write program finished and Close log file
		/// </summary>
		public void Finish ()
		{
			Write (program_name + " Finished");
			log_stream.Close ();
		}

		/// <summary> 
		/// Write blank string to both log file and WScript.Echo (console)
		/// </summary> 
		public void Write ()
		{

			Console.WriteLine ();
			log_stream.WriteLine ();
			log_stream.Flush ();
		}

		/// <summary> 
		/// Write string to both log file and WScript.Echo (console)
		/// </summary> 
		/// <param name="string">The string to write</param> 
		public void Write (string str)
		{

			// Close log file with a new day

			if (current_date != DateTime.Today)
			{
				current_date = DateTime.Today;
				Write (program_name + " log Closed");
				log_stream.Close ();

				// Start a new log file with the new day
				NewLog ();
				Write (program_name + " log restarted");
			}
			if (1 < str.Length)
				str = datetime.strDateTime () + " " + str;
			Console.WriteLine (str);
			log_stream.WriteLine (str);
			log_stream.Flush ();
		}

		// Private Methods

		void InitLog (string directory)
		///////////////////////////////////////////////////////////////////////////////
		//
		// Initialize log file and Console
		//
		///////////////////////////////////////////////////////////////////////////////
		{

			directory_name = directory;
			program_name = Path.GetFileNameWithoutExtension (AppDomain.CurrentDomain.FriendlyName);

			Console.Title = program_name;
			Console.WindowHeight = Console.LargestWindowHeight - 10;
			Console.WindowWidth = Console.LargestWindowWidth - 40;
			Console.BackgroundColor = ConsoleColor.Blue;
			Console.ForegroundColor = ConsoleColor.Yellow;
			Console.Clear ();
			Console.CursorSize = 100;

			NewLog ();
			Write (program_name + " Started");
		}

		void NewLog ()
		///////////////////////////////////////////////////////////////////////////////
		//
		// Erase log files over a week old and start new log
		//
		///////////////////////////////////////////////////////////////////////////////
		{
			DateTime tdate;

			current_date = DateTime.Today;
			tdate = current_date.AddDays (-14);
			for (int i = 0; i < 7; ++i)
			{
				try
				{
					File.Delete (name (tdate));
				}
				catch { }
				tdate = tdate.AddDays (1);
			}

			log_stream = File.AppendText (name (current_date));
			Write ("_____");
			Write ("BaseDirectory: " + AppDomain.CurrentDomain.BaseDirectory);
			Write ("LogDirectory: " + directory_name);
		}

		string name (DateTime date)
		///////////////////////////////////////////////////////////////////////////////
		//
		// creat a standard name based on the date
		//
		///////////////////////////////////////////////////////////////////////////////
		{
			return directory_name + program_name + "-" + datetime.strDate (date) + ".log";
		}

		// Public Static Methods

		public static void Beep ()
		///////////////////////////////////////////////////////////////////////////////
		//
		// A pleasing melodic beep
		//
		///////////////////////////////////////////////////////////////////////////////
		{
			Console.Beep (880, 10);
			Console.Beep (440, 10);
		}

		public static string Pause ()
		///////////////////////////////////////////////////////////////////////////////
		//
		// pause for console run without command line when displaying an error message
		//
		///////////////////////////////////////////////////////////////////////////////
		{

			Beep ();
			Console.WriteLine ();
			Console.Write ("Type the ENTER key when ready...");
			var str = Console.ReadLine ();
			Console.WriteLine ();
			return str;
		}

		public static string Pause (string str)
		///////////////////////////////////////////////////////////////////////////////
		//
		// pause for console run without command line when displaying an error message
		//
		///////////////////////////////////////////////////////////////////////////////
		{

			Console.WriteLine ();
			Console.WriteLine (str);
			return Pause ();
		}

		public static string Prompt (string str)
		///////////////////////////////////////////////////////////////////////////////
		//
		// prompt for input
		//
		///////////////////////////////////////////////////////////////////////////////
		{

			Console.WriteLine ();
			Console.Write (str);
			str = Console.ReadLine ();
			Console.WriteLine ();
			return str;
		}
	}
}
