﻿using System;
using System.Globalization;

/////////////////////////////////////////////////////////////////////////
//
// ISO generic datetime functions 2001-06-01 12:13:13.222
//
/////////////////////////////////////////////////////////////////////////
namespace DateTimeISO
{
	class datetime
	{
		static string[] formats = {"yyyy-MM-dd HH:mm:ss.fff", 
                   "yyyy-MM-dd", 
                   "HH:mm:ss.fff"};

		/////////////////////////////////////////////////////////////////////////
		/// <summary> 
		/// Return ISO format time string for DateTime.Now 
		/// </summary> 
		/////////////////////////////////////////////////////////////////////////
		public static string strTime ()
		{
			return strTime (DateTime.Now);
		}

		/////////////////////////////////////////////////////////////////////////
		/// <summary> 
		/// Return ISO format date string for DateTime.Now
		/// </summary> 
		/////////////////////////////////////////////////////////////////////////
		public static string strDate ()
		{
			return strDate (DateTime.Now);
		}

		/////////////////////////////////////////////////////////////////////////
		/// <summary> 
		/// Return ISO format datetime string for DateTime.Now
		/// </summary> 
		/////////////////////////////////////////////////////////////////////////
		public static string strDateTime ()
		{
			return strDateTime (DateTime.Now);
		}

		/////////////////////////////////////////////////////////////////////////
		/// <summary> 
		/// Return ISO format time string for TimeSpan value
		/// </summary> 
		/// <param name="timespan">The TimeSpan value to convert to string</param> 
		/////////////////////////////////////////////////////////////////////////
		public static string strTime (TimeSpan timespan)
		{
			return timespan.Hours.ToString ("00") + ":" + timespan.Minutes.ToString ("00")
					+ ":" + timespan.Seconds.ToString ("00") + "."
					+ timespan.Milliseconds.ToString ("000");
		}

		/////////////////////////////////////////////////////////////////////////
		/// <summary> 
		/// Return ISO format datetime string for DateTime value
		/// </summary> 
		/// <param name="datetime">The DateTime value to convert to string</param> 
		/////////////////////////////////////////////////////////////////////////
		public static string strTime (DateTime datetime)
		{
			return datetime.ToString (formats[2]);
		}

		/////////////////////////////////////////////////////////////////////////
		/// <summary> 
		/// Return ISO format datetime string for DateTime value
		/// </summary> 
		/// <param name="datetime">The DateTime value to convert to string</param> 
		/////////////////////////////////////////////////////////////////////////
		public static string strDate (DateTime datetime)
		{
			return datetime.ToString (formats[1]);
		}

		/////////////////////////////////////////////////////////////////////////
		/// <summary> 
		/// Return ISO format datetime string for DateTime value
		/// </summary> 
		/// <param name="datetime">The DateTime value to convert to string</param> 
		/////////////////////////////////////////////////////////////////////////
		public static string strDateTime (DateTime datetime)
		{
			return datetime.ToString (formats[0]);
		}

		/////////////////////////////////////////////////////////////////////////
		/// <summary> 
		/// Return DateTime value from ISO format date and/or time string
		/// </summary> 
		/// <param name="str">The string to convert to DateTime value</param> 
		/////////////////////////////////////////////////////////////////////////
		public static DateTime dateTime (string str)
		{
			DateTime dateValue;

			DateTime.TryParseExact (str, formats, new CultureInfo ("en-US"),
						   DateTimeStyles.None, out dateValue);
			return dateValue;
		}
	}
}