﻿#if DEBUG
#warning Debug build
#endif

using System;
using System.IO;
using System.Collections.Generic;

using ConsoleLog;
using DateTimeISO;
using STemplate;

namespace STemplateA
{
	///////////////////////////////////////////////////////////////////////////
	//
	// STemplateA - Analyze template to create skeleton code
	// error checking later?
	//
	///////////////////////////////////////////////////////////////////////////
	class S_TemplateA
	{
		public static Console_Log log;
		public static S_Template template;

		public static string element_start = "{";
		public static string element_stop = "}";
		public static string section_start = "{{";
		public static string section_stop = "}}";

		// List of sections and elements found
		static Stack<string> sectionList = new Stack<string> ();
		static bool sections = false;

		///////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////
		static void Main (string[] args)
		{
			log = new Console_Log ();
			template = new S_Template ();

			Console.WriteLine ();
			Console.WriteLine ("Simple String Template Analyzer");
			Console.WriteLine ();

			if (1 != args.Length)
			{
				Console.WriteLine ("Missing expected file argument!");
				return;
			}

#if !DEBUG
			try
			{
#endif

			ProcessFile (args[0]);

#if !DEBUG
			}
			catch (Exception ex)
			{
				log.Write (ex.ToString ());
			}
#endif
			Console_Log.Pause ("Finished!");
			log.Finish ();
		}

		///////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////
		static void ProcessFile (string filename)
		{
			string str = null;
			int done = 0;

			log.Write ("Reading " + filename);
			string analyze_file = File.ReadAllText (filename);
			template.Read ("STemplateA.txt");
			template.Replace ("{template_analyze_file}", filename);
			sectionList.Push ("@");

			// find sections
			for (int section_begin = 0; ; )
			{
				string section_str = Symbol (analyze_file, section_start, section_stop, done, out section_begin);
				if (0 > section_begin)
					section_begin = analyze_file.Length;

				//find all current section elements
				for (; ; )
				{
					int next_element;
					str = Symbol (analyze_file, element_start, element_stop, done, out next_element);
					if (0 > next_element || next_element >= section_begin)
					{
						done = section_begin + section_str.Length;
						break;
					}
					// is this element already in the list
					string tstr = sectionList.Pop ();
					if (!tstr.Contains (str))
						tstr += " " + str;
					sectionList.Push (tstr);
					done = next_element + str.Length;
				}

				// is this the end of the file?
				if ("" == section_str)
				{
					section_str = "@";
					for (int i = sectionList.Count; i > 1; --i)
					{
						str = sectionList.Pop ();
						log.Write ("No ending section " + str);
					}
				}
				// same name means end of section
				if (sectionList.Peek ().StartsWith (section_str))
				{
					str = sectionList.Pop ();
					string[] a = str.Split (new Char[] { ' ' });
					if ("@" == section_str)
					{
						if (sections)
							template.SectionEnd ();
						else
							template.SectionRemove ("{{template_analyze_section}}");

						if (1 == a.Length)
							template.SectionRemove ("{{template_analyze_main_replace}}");
						else
						{
							for (int i = 1; i < a.Length; ++i)
							{
								template.SectionBegin ("{{template_analyze_main_replace}}");
								template.Replace ("{template_analyze_text1}", a[i]);
							}
							template.SectionEnd ();
						}
						break;
					}
					else
					{
						template.SectionBegin ("{{template_analyze_section}}");
						template.Replace ("{template_analyze_section_name}", section_str);
						if (1 == a.Length)
							template.SectionRemove ("{{template_analyze_section_replace}}");
						else
						{
							for (int i = 1; i < a.Length; ++i)
							{
								template.SectionBegin ("{{template_analyze_section_replace}}");
								template.Replace ("{template_analyze_text2}", a[i]);
							}
							template.SectionEnd ();
						}
					}
				}
				else
				{
					sections = true;
					sectionList.Push (section_str);
				}
			}
			template.Write (filename + ".new");
		}

		///////////////////////////////////////////////////////////////////////
		//
		// find a symbol delimited by start and stop of 32 characters or less
		//
		///////////////////////////////////////////////////////////////////////
		static string Symbol (string str, string delimit_start, string delimit_stop, int done, out int index)
		{
			string symbol = "";
			index = -1;
			while (done < str.Length)
			{
				index = str.IndexOf (delimit_start, done);
				if (0 > index)
					break;
				int first = index + delimit_start.Length;
				int end = str.IndexOf (delimit_stop, first);
				if (0 > end)
					break;
				int l = end - first;
				if (1 > l || 32 < l)
				{
					done = first;
					continue;
				}
				int i;
				for (i = 0; i < l; ++i)
				{
					if (!char.IsLetterOrDigit (str[first + i]))
						break;
				}
				if (i < l)
				{
					done = first;
					continue;
				}
				symbol = str.Substring (index, l + delimit_start.Length + delimit_stop.Length);
				break;
			}
			if ("" == symbol)
				index = -1;
			return symbol;
		}
	}
}
